@@ -10,23 +10,21 @@ import (
 	"strconv"
 	"strings"
 
-	"gitlab.com/gitlab-org/cli/api"
-	"gitlab.com/gitlab-org/cli/pkg/iostreams"
+	"gitlab.com/gitlab-org/cli/internal/api"
+	"gitlab.com/gitlab-org/cli/internal/iostreams"
+	"gitlab.com/gitlab-org/cli/internal/utils"
 
 	"github.com/mgutz/ansi"
 
 	surveyCore "github.com/AlecAivazis/survey/v2/core"
-	"gitlab.com/gitlab-org/cli/commands"
-	"gitlab.com/gitlab-org/cli/commands/alias/expand"
-	"gitlab.com/gitlab-org/cli/commands/cmdutils"
-	"gitlab.com/gitlab-org/cli/commands/help"
-	"gitlab.com/gitlab-org/cli/commands/hooks"
-	"gitlab.com/gitlab-org/cli/commands/update"
+	"gitlab.com/gitlab-org/cli/internal/cmdutils"
+	"gitlab.com/gitlab-org/cli/internal/commands"
+	"gitlab.com/gitlab-org/cli/internal/commands/alias/expand"
+	"gitlab.com/gitlab-org/cli/internal/commands/help"
+	"gitlab.com/gitlab-org/cli/internal/commands/update"
 	"gitlab.com/gitlab-org/cli/internal/config"
 	"gitlab.com/gitlab-org/cli/internal/run"
-	"gitlab.com/gitlab-org/cli/pkg/glinstance"
-	"gitlab.com/gitlab-org/cli/pkg/tableprinter"
-	"gitlab.com/gitlab-org/cli/pkg/utils"
+	"gitlab.com/gitlab-org/cli/internal/tableprinter"
 
 	"github.com/spf13/cobra"
 )
@@ -38,80 +36,81 @@ var (
 	commit string
 	// platform is set dynamically at build
 	platform = runtime.GOOS
+
+	// debugMode is set dynamically at build and can be overridden by
+	// the configuration file or environment variable
+	// sets to "true" or "false" or "1" or "0" as string
+	debugMode = "false"
 )
 
-// debug is set dynamically at build and can be overridden by
-// the configuration file or environment variable
-// sets to "true" or "false" or "1" or "0" as string
-var debugMode = "false"
-
-// debug is parsed boolean of debugMode
-var debug bool
-
 func main() {
-	debug = debugMode == "true" || debugMode == "1"
-
-	cmdFactory := cmdutils.NewFactory(iostreams.Init(), true)
-
-	cfg, err := cmdFactory.Config()
+	// Initialize configuration
+	cfg, err := config.Init()
 	if err != nil {
-		cmdFactory.IO().Logf("failed to read configuration:  %s\n", err)
+		fmt.Fprintf(os.Stderr, "failed to read configuration:  %s\n", err)
 		os.Exit(2)
 	}
 
-	api.SetUserAgent(version, platform, runtime.GOARCH)
-	maybeOverrideDefaultHost(cmdFactory, cfg)
-
-	if !cmdFactory.IO().ColorEnabled() {
-		surveyCore.DisableColor = true
-	} else {
-		// Override survey's choice of color for default values
-		// For default values for e.g. `Input` prompts, Survey uses the literal "white" color,
-		// which makes no sense on dark terminals and is literally invisible on light backgrounds.
-		// This overrides Survey to output a gray color for 256-color terminals and "default" for basic terminals.
-		surveyCore.TemplateFuncsWithColor["color"] = func(style string) string {
-			switch style {
-			case "white":
-				if cmdFactory.IO().Is256ColorSupported() {
-					return fmt.Sprintf("\x1b[%d;5;%dm", 38, 242)
-				}
-				return ansi.ColorCode("default")
-			default:
-				return ansi.ColorCode(style)
-			}
-		}
-	}
-
-	rootCmd := commands.NewCmdRoot(cmdFactory, version, commit)
-
 	// Set Debug mode from config if not previously set by debugMode
+	debug := debugMode == "true" || debugMode == "1"
 	if !debug {
 		debugModeCfg, _ := cfg.Get("", "debug")
 		debug = debugModeCfg == "true" || debugModeCfg == "1"
 	}
 
-	if pager, _ := cfg.Get("", "glab_pager"); pager != "" {
-		cmdFactory.IO().SetPager(pager)
-	}
+	// Initialize factory and iostreams
+	cmdFactory := cmdutils.NewFactory(
+		iostreams.New(
+			iostreams.WithStdin(os.Stdin, iostreams.IsTerminal(os.Stdin)),
+			iostreams.WithStdout(iostreams.NewColorable(os.Stdout), iostreams.IsTerminal(os.Stdout)),
+			iostreams.WithStderr(iostreams.NewColorable(os.Stderr), iostreams.IsTerminal(os.Stderr)),
+			iostreams.WithPagerCommand(iostreams.PagerCommandFromEnv()),
 
-	if promptDisabled, _ := cfg.Get("", "no_prompt"); promptDisabled != "" {
-		cmdFactory.IO().SetPrompt(promptDisabled)
-	}
+			// overwrite pager from env if set via config
+			func(i *iostreams.IOStreams) {
+				if pager, _ := cfg.Get("", "glab_pager"); pager != "" {
+					i.SetPager(pager)
+				}
+			},
 
-	if forceHyperlinks := os.Getenv("FORCE_HYPERLINKS"); forceHyperlinks != "" && forceHyperlinks != "0" {
-		cmdFactory.IO().SetDisplayHyperlinks("always")
-	} else if displayHyperlinks, _ := cfg.Get("", "display_hyperlinks"); displayHyperlinks == "true" {
-		cmdFactory.IO().SetDisplayHyperlinks("auto")
-	}
+			// configure hyperlink display
+			func(i *iostreams.IOStreams) {
+				if enabled, found := utils.IsEnvVarEnabled("FORCE_HYPERLINKS"); found {
+					if enabled {
+						i.SetDisplayHyperlinks("always")
+					}
 
+					return
+				}
+
+				if displayHyperlinks, _ := cfg.Get("", "display_hyperlinks"); displayHyperlinks == "true" {
+					i.SetDisplayHyperlinks("auto")
+				}
+			},
+
+			// configure prompt
+			func(i *iostreams.IOStreams) {
+				if promptDisabled, _ := cfg.Get("", "no_prompt"); promptDisabled != "" {
+					i.SetPrompt(promptDisabled)
+				}
+			},
+		),
+		true,
+		cfg,
+		api.BuildInfo{Version: version, Commit: commit, Platform: platform, Architecture: runtime.GOARCH},
+	)
+
+	setupSurveyCore(cmdFactory.IO())
+
+	// Setup command
 	var expandedArgs []string
 	if len(os.Args) > 0 {
 		expandedArgs = os.Args[1:]
 	}
-
+	rootCmd := commands.NewCmdRoot(cmdFactory)
 	cmd, _, err := rootCmd.Traverse(expandedArgs)
 
-	checkForTelemetryHook(cfg, cmdFactory, cmd)
+	setupTelemetryHook(cfg, cmdFactory, cmd)
 
 	if err != nil || cmd == rootCmd {
 		originalArgs := expandedArgs
@@ -176,30 +175,9 @@ func main() {
 	var argCommand string
 	if expandedArgs != nil {
 		argCommand = expandedArgs[0]
-	} else {
-		argCommand = ""
 	}
-
-	shouldCheck := false
-
-	// GLAB_CHECK_UPDATE has higher priority than the check_update configuration value
-	if envVal, ok := os.LookupEnv("GLAB_CHECK_UPDATE"); ok {
-		if checkUpdate, err := strconv.ParseBool(envVal); err == nil {
-			shouldCheck = checkUpdate
-		}
-	} else {
-		// Fall back to config value if env var not set
-		if checkUpdate, _ := cfg.Get("", "check_update"); checkUpdate != "" {
-			if parsed, err := strconv.ParseBool(checkUpdate); err == nil {
-				shouldCheck = parsed
-			}
-		}
-	}
-
-	if shouldCheck {
-		if err := update.CheckUpdate(cmdFactory, version, true, argCommand); err != nil {
-			printError(cmdFactory.IO(), err, rootCmd, debug)
-		}
+	if !update.ShouldSkipUpdate(argCommand) {
+		checkForUpdate(cmdFactory, rootCmd, debug)
 	}
 }
 
@@ -233,26 +211,60 @@ func printError(streams *iostreams.IOStreams, err error, cmd *cobra.Command, deb
 	}
 }
 
-func maybeOverrideDefaultHost(f cmdutils.Factory, cfg config.Config) {
-	baseRepo, err := f.BaseRepo()
-	if err == nil {
-		glinstance.OverrideDefault(baseRepo.RepoHost())
-	}
-
-	// Fetch the custom host config from env vars, then local config.yml, then global config,yml.
-	customGLHost, _ := cfg.Get("", "host")
-	if customGLHost != "" {
-		if utils.IsValidURL(customGLHost) {
-			var protocol string
-			customGLHost, protocol = glinstance.StripHostProtocol(customGLHost)
-			glinstance.OverrideDefaultProtocol(protocol)
+func setupSurveyCore(io *iostreams.IOStreams) {
+	if !io.ColorEnabled() {
+		surveyCore.DisableColor = true
+	} else {
+		// Override survey's choice of color for default values
+		// For default values for e.g. `Input` prompts, Survey uses the literal "white" color,
+		// which makes no sense on dark terminals and is literally invisible on light backgrounds.
+		// This overrides Survey to output a gray color for 256-color terminals and "default" for basic terminals.
+		surveyCore.TemplateFuncsWithColor["color"] = func(style string) string {
+			switch style {
+			case "white":
+				if io.Is256ColorSupported() {
+					return fmt.Sprintf("\x1b[%d;5;%dm", 38, 242)
+				}
+				return ansi.ColorCode("default")
+			default:
+				return ansi.ColorCode(style)
+			}
 		}
-		glinstance.OverrideDefault(customGLHost)
 	}
 }
 
-func checkForTelemetryHook(cfg config.Config, f cmdutils.Factory, cmd *cobra.Command) {
-	if hooks.IsTelemetryEnabled(cfg) {
-		cobra.OnFinalize(hooks.AddTelemetryHook(f, cmd))
+func setupTelemetryHook(cfg config.Config, f cmdutils.Factory, cmd *cobra.Command) {
+	if isTelemetryEnabled(cfg) {
+		cobra.OnFinalize(addTelemetryHook(f, cmd))
 	}
 }
+
+func checkForUpdate(f cmdutils.Factory, rootCmd *cobra.Command, debug bool) {
+	if !isUpdateCheckEnabled(f) {
+		return
+	}
+
+	if err := update.CheckUpdate(f, true); err != nil {
+		printError(f.IO(), err, rootCmd, debug)
+	}
+}
+
+func isUpdateCheckEnabled(f cmdutils.Factory) bool {
+	if enabled, found := utils.IsEnvVarEnabled("GLAB_CHECK_UPDATE"); found {
+		return enabled
+	}
+
+	val, err := f.Config().Get("", "check_update")
+	// WARN: I return true here since I think we should always check for updates
+	// and an error likely indicates that the value wasn't found in the config.
+	if err != nil {
+		return true
+	}
+
+	checkUpdate, err := strconv.ParseBool(val)
+	if err != nil {
+		f.IO().Logf("ERROR: Could not parse config value %q: %s", "check_update", err)
+	}
+
+	return checkUpdate
+}
